---
docker_daemon_debug: true
# Use the OpenStack infra's Dockerhub mirror.
docker_registry_mirrors:
  - "http://{{ zuul_site_mirror_fqdn }}:8082/"

kolla_source_url: "{{ ansible_env.PWD ~ '/' ~ zuul.projects['opendev.org/openstack/kolla'].src_dir }}"
kolla_source_version: "{{ zuul.projects['opendev.org/openstack/kolla'].checkout }}"
kolla_ansible_source_url: "{{ ansible_env.PWD ~ '/' ~ zuul.projects['opendev.org/openstack/kolla-ansible'].src_dir }}"
kolla_ansible_source_version: "{{ zuul.projects['opendev.org/openstack/kolla-ansible'].checkout }}"
kolla_ansible_requirements_yml: "/tmp/kolla-ansible-requirements.yml"
kolla_openstack_logging_debug: True
pip_upper_constraints_file: "/tmp/upper-constraints.txt"

# Nested virtualisation is not working well in CI currently. Force the use of
# QEMU.
libvirt_vm_engine: "qemu"

# Use the CI infra's PyPI mirror.
pip_local_mirror: true
pip_index_url: "http://{{ zuul_site_mirror_fqdn }}/pypi/simple"
pip_trusted_hosts:
  - "{{ zuul_site_mirror_fqdn }}"

# Try with only a single VCPU, word on the street is that QEMU doesn't play
# nicely with more than one.
seed_vm_vcpus: 1

# Reduce the memory footprint of the seed VM.
seed_vm_memory_mb: "{{ 1 * 1024 }}"

# Use cirros rather than CentOS for the VM.
seed_bootstrap_user: cirros
seed_vm_root_image: /opt/cache/files/cirros-0.5.3-x86_64-disk.img

{% if seed_vm_boot_firmware is defined %}
seed_vm_boot_firmware: "{{ seed_vm_boot_firmware }}"
{% endif %}
{% if seed_vm_machine is defined %}
seed_vm_machine: "{{ seed_vm_machine }}"
{% endif %}

# Cirros doesn't load cdom drivers by default.
seed_vm_configdrive_device: disk

# Cirros is Debian family, but doesn't support path globs in
# /etc/network/interfaces.
configdrive_os_family: Debian
configdrive_debian_network_interfaces_supports_glob: false

# NOTE(mgoddard): CentOS 8 removes interfaces from their bridge during ifdown,
# and removes the bridge if there are no interfaces left. When Kayobe bounces
# veth links plugged into the bridge, it causes the bridge which has the IP we
# are using for SSH to be removed. Use a dummy interface.
aio_bridge_ports:
  - dummy1

overcloud_dib_build_host_images: False
