---
docker_daemon_debug: true
# Use the OpenStack infra's Dockerhub mirror.
docker_registry_mirrors:
  - "http://{{ zuul_site_mirror_fqdn }}:8082/"

kolla_docker_namespace: "openstack.kolla"
# use the published images from a site mirror of quay.io
kolla_docker_registry: "{{ zuul_site_mirror_fqdn }}:4447"
# NOTE(mgoddard): The kolla repository is copied to /tmp/kolla and made
# readable by the stack user.
kolla_source_url: "/tmp/kolla"
kolla_source_version: "{{ zuul.projects['opendev.org/openstack/kolla'].checkout }}"
kolla_ansible_source_url: "{{ ansible_env.PWD ~ '/' ~ zuul.projects['opendev.org/openstack/kolla-ansible'].src_dir }}"
kolla_ansible_source_version: "{{ zuul.projects['opendev.org/openstack/kolla-ansible'].checkout }}"
kolla_ansible_requirements_yml: "/tmp/kolla-ansible-requirements.yml"
kolla_openstack_logging_debug: True
pip_upper_constraints_file: "/tmp/upper-constraints.txt"

# Use the CI infra's PyPI mirror.
pip_local_mirror: true
pip_index_url: "http://{{ zuul_site_mirror_fqdn }}/pypi/simple"
pip_trusted_hosts:
  - "{{ zuul_site_mirror_fqdn }}"

# NOTE(mgoddard): CentOS 8 removes interfaces from their bridge during ifdown,
# and removes the bridge if there are no interfaces left. When Kayobe bounces
# veth links plugged into the bridge, it causes the bridge which has the IP we
# are using for SSH to be removed. Use a dummy interface.
aio_bridge_ports:
  - dummy1

# Build seed deployment images (IPA) with extra-hardware element
ipa_build_images: {{ build_images }}
ipa_build_dib_elements_extra:
  # extra-hardware is currently failing on Ubuntu
  - "{% raw %}{{ 'extra-hardware' if os_distribution != 'ubuntu' else '' }}{% endraw %}"
  - "openstack-ci-mirrors"

# Workaround for limited tmpfs space in CI
ipa_build_dib_env_extra:
  DIB_NO_TMPFS: 1

# Build overcloud host image.
overcloud_dib_build_host_images: {{ build_images }}
overcloud_dib_elements_extra:
  - "openstack-ci-mirrors"

# NOTE(bbezak): Kolla does not build CentOS Stream 9 container images.
# Using Rocky Linux 9 images on CentOS Stream 9 in CI.
kolla_base_distro: "{% raw %}{{ 'rocky' if os_distribution == 'centos' else os_distribution }}{% endraw %}"
