#jinja2: trim_blocks: True,lstrip_blocks: True

{% for line in junos_switch_config %}
{{ line }}
{% endfor %}

{% if junos_switch_interface_config %}
interfaces {
{% for interface, config in junos_switch_interface_config.items() %}
    {{ interface }} {
{% if config.description is defined %}
        description "{{ config.description }}";
{% endif %}
{% for line in config.config %}
{{ line }}
{% endfor %}
    }
{% endfor %}
}
{% endif %}
